
/*
 * This class contains constants which in some cases are used throughout the 
 * mass equation calculations. The physics constants though are only used to
 * calculate the derived mass element constant, which is one of the 
 * introductory parts of the mass equations. The mass element has type kg.
 *
 * NOTE: I've used the most recent precise values i could find. 
 */
public class Constants {

    public static double plancksConstant() {

	return 1.05455716818 * Math.pow(10.0, -34.0);
    }

    public static double lightVelocity() {
	
	return 2.99792458 * Math.pow(10.0, 8.0);
    }

    /*
     * NOTE: This value is incorrect in (p4, 1982 formula) 
     */
    public static double gravitationConstant() {

	return 6.6742 * Math.pow(10.0, -11.0);
    }

    /*
     * Not used yet.
     */
    public static double influenceConstant() {

	return 8.8542 * Math.pow(10.0, -12.0);
    }

    /*
     * Not used yet.
     */
    public static double inductionConstant() {

	return 1.2566 * Math.pow(10.0, -6.0);
    }

    public static double vacuumWaveResistance() {

	return 376.73037659;
    }

    public static double geometricalConstant() {

	return (1+Math.sqrt(5))/2;
    }    

    /*
     * The Mass Constant - EQUATION VI (p4, 1982 formula) 
    */
    public static double massElement() {

	return (Math.pow(Math.PI, (1.0/4.0)) * 
		Math.pow(3*Math.PI*gravitationConstant()*plancksConstant(), (1.0/3.0)) *
		Math.sqrt(plancksConstant()/(3*lightVelocity()*gravitationConstant())));
    }

    /*
     * alpha - middle of page (p4, 1982 formula)
     */
    public static double alpha() {

	return 1/137.0360085;
    }

    /*
     * eta - constant version of eta in EQUATIONS V (p3, 1982 formula)
     *
     * NOTE: The non-constant versions can be found in the EQ class.
     */
    public static double eta() {

	return Math.PI/Math.pow((Math.pow(Math.PI, 4.0)+4), (1.0/4.0));
    } 

    /*
     * theta - constant version of theta in EQUATIONS V (p3, 1982 formula)
     *
     * NOTE: The non-constant versions can be found in the EQ class.
     */
    public static double theta() {

	return (5*eta()) + (2*Math.sqrt(eta())) + 1;
    } 

    /*
     * alpha+ - one of EQUATIONS VIII (p4, 1982 formula)
     */
    public static double alphaPlus() {

	return (Math.pow(eta(), (1.0/6.0))/Math.pow(eta(), 2.0) *
		(1 - theta()*Math.pow((2*(1-Math.sqrt(eta())))/(eta()*(1+Math.sqrt(eta()))), 2.0)*Math.sqrt(2*eta())) - 1);
    } 

    /*
     * alpha- - one of EQUATIONS VIII (p4, 1982 formula)
     */
    public static double alphaMinus() {

	return ((alphaPlus()+1)*eta()) - 1;
    } 

}
