
/*
 * This class deals with equations related to determining the value of the Quantum 
 * Number EQUATIONS IX-X (p5, 1982 formula) and B8-B14 (p12, 1989 formula). Note that
 * equation N3 is the one given in the 1989 formula. 
 */
public class QN {

    /*
     * eta - one of EQUATIONS V (p3, 1982 formula)
     *
     * NOTE: eta(1,0) = eta according to the 1989 formula. The constant version 
     * of the function eta, eta(1,0) is given in the Constants class.
     */
    public static double eta(double k, double q) {

	return Math.PI/Math.pow(Math.pow(Math.PI, 4)+((4+k)*Math.pow(q, 4.0)), (1.0)/(4.0));
    }

    /*
     * theta - one of EQUATIONS V (p3, 1982 formula)
     *
     * NOTE: theta(1,0) = theta according to the 1989 formula. The constant version 
     * of the function theta, theta(1,0) is given in the Constants class.
     */
    public static double theta(double k, double q) {

	return (5*eta(k, q)) + (2*Math.sqrt(eta(k, q))) + 1;
    } 

    /*
     * N1 - one of EQUATIONS IX (p5, 1982 formula)
     */
    public static double N1(double k, double q) {

	return 0.5*(1+Math.sqrt(eta(k,q)));
    }

    /*
     * N2 - one of EQUATIONS IX (p5, 1982 formula)
     */
    public static double N2(double k, double q) {

	return (2.0/3.0)*(1/eta(k,q));
    }

    /*
     * N3 - EQUATION B8 (p12, 1989 formula)
     */
    public static double N3(double k, double q) {

	// NOTE: What does the symbol small u stand for?
	// it is not given in either the 1982 nor 1989 formula.
	// I assume this is the reason N3 is inaccurate.

	return (Math.pow(Math.E, (k-1)*
			 (1-(Math.PI*((1-eta(k, q))/(1+Math.sqrt(eta(1, q))))*
			     (1-(/*u???*/1*((eta(1, q))/(theta(1, q)))*
				 ((1-Constants.alphaMinus())/(Constants.alphaPlus()))*
				 Math.pow((1-Math.sqrt(Constants.eta())), 2))))) -
			 ((2/(3*Math.PI*Math.E))*
			  Math.pow((1-Math.sqrt(Constants.eta())), 2)*
			  ((6.0*Math.pow(Math.PI, 2)*(Math.pow(Math.E, 2)/Constants.theta())*
			    ((1+Math.sqrt(eta(1, q)))/(1-Constants.eta())))-1)))/(k/2.0));
    }

    /*
     * N4 - EQUATION B9 (p12, 1989 formula)
     */
    public static double N4(double k, double q) {

	return (4/k)*(1+(q*(k-1)));
    }

    /*
     * A - EQUATION B11 (p12, 1989 formula)
     */
    private static double A() {

	return ((8/Constants.eta())*
		(1-(Constants.alphaMinus()/Constants.alphaPlus()))*
		(1-((3*Constants.eta())/4)));
    }

    /*
     * N - EQUATION B12 (p12, 1989 formula)
     */
    private static double N(double k) {

	return (Q1(k)+Q2(k)+Q3(k)+Q4(k)+(k*(Math.pow(-1, k))*Math.pow(2, Math.pow(k, 2)-1)));
    }

    /*
     * N' - EQUATION B14 (p12, 1989 formula)
     */
    private static double diffN(double k) {

	return (Q1(k)+Q2(k)+Q3(k)+Q4(k)-(2*k)-1);
    }

    /*
     * N5 - EQUATION B10 (p12, 1989 formula)
     */
    public static double N5(double k, double q) {

	return (A()*(1+(k*(k-1)*
			Math.pow(2, Math.pow(k, 2)+3)*N(k)*
			A()*Math.pow((1-Math.sqrt(eta(k, q)))/(1+Math.sqrt(eta(k, q))), 2))));
    }

    /*
     * N6 - EQUATION B10 (p12, 1989 formula)
     */
    public static double N6(double k, double q) {

	return (((2*k)/(Math.PI*Math.E*Constants.theta()))*
		((Math.sqrt(k)*(Math.pow(k, 2)-1)*(N(k)/Math.sqrt(eta(k, 1)))*
		  (q-((1-q)*(diffN(k)/(Q1(k)*Math.sqrt(eta(k, 1)))))))+Math.pow(-1, k+1))*
		Constants.eta()*(1-(Constants.alphaMinus()/Constants.alphaPlus()))*
		Math.pow(4*((1-Math.sqrt(Constants.eta()))/(1+Math.sqrt(Constants.eta()))), 2)*Q4(k));
    }


    /*
     * Small Function needed for the below equations
     */
    private static double s(double k) {

	return Math.pow(k, 2) + 1;
    }

    /*
     * Q1 - one of EQUATIONS X (p5, 1982 formula)
     * 
     * NOTE: in the 1989 formula this is refered to as Qn
    */
    public static double Q1(double k) {
	
	return 3 * Math.pow(2, s(k)-2);
    }

    /*
     * Q2 - one of EQUATIONS X (p5, 1982 formula)
     * 
     * NOTE: in the 1989 formula this is refered to as Qm
     */
    public static double Q2(double k) {
	
	return Math.pow(2, s(k))-1;
    }

    /*
     * Q3 - one of EQUATIONS X (p5, 1982 formula)
     * 
     * NOTE: in the 1989 formula this is refered to as Qp
     */
    public static double Q3(double k) {
	
	return Math.pow(2, s(k))+(2*Math.pow(-1, k));
    }

    /*
     * Q4 - one of EQUATIONS X (p5, 1982 formula)
     * 
     * NOTE: in the 1989 formula this is refered to as Qsigma
     */
    public static double Q4(double k) {
	
	return Math.pow(2, s(k)-1)-1;
    }
}
